﻿using MyFirstApp.Data;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Search Contract item template is documented at http://go.microsoft.com/fwlink/?LinkId=234240

namespace MyFirstApp
{
	/// <summary>
	/// This page displays search results when a global search is directed to this application.
	/// </summary>
	public sealed partial class SearchResultsPage : MyFirstApp.Common.LayoutAwarePage
	{

		public SearchResultsPage()
		{
			this.InitializeComponent();
		}
		string searchText;

		/// <summary>
		/// Populates the page with content passed during navigation.  Any saved state is also
		/// provided when recreating a page from a prior session.
		/// </summary>
		/// <param name="navigationParameter">The parameter value passed to
		/// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
		/// </param>
		/// <param name="pageState">A dictionary of state preserved by this page during an earlier
		/// session.  This will be null the first time a page is visited.</param>
		protected override async void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
		{
			searchText = navigationParameter as String;

			// TODO: Application-specific searching logic.  The search process is responsible for
			//       creating a list of user-selectable result categories:

			var groups = await SampleDataSource.GetGroups("AllGroups");

			// tutti
			var filterList = new List<Filter>();
			filterList.Add(new Filter("Tutti", groups.SelectMany(x => x.Items).Where(x => x.Title.ToLower().Contains(searchText.ToLower())).ToList(), true));

			// gruppi
			foreach (var item in groups)
			{
				filterList.Add(new Filter(item.Title, item.Items.Where(x => x.Title.ToLower().Contains(searchText.ToLower())), true));
			}

			// Communicate results through the view model
			this.DefaultViewModel["QueryText"] = '\u201c' + searchText + '\u201d';
			this.DefaultViewModel["Filters"] = filterList;
			this.DefaultViewModel["ShowFilters"] = filterList.Count > 1;

			if (filtersViewSource.View != null)
				filtersViewSource.View.MoveCurrentTo(filterList[0]);
		}

		/// <summary>
		/// Invoked when a filter is selected using the ComboBox in snapped view state.
		/// </summary>
		/// <param name="sender">The ComboBox instance.</param>
		/// <param name="e">Event data describing how the selected filter was changed.</param>
		async void Filter_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			// Determine what filter was selected
			var selectedFilter = e.AddedItems.FirstOrDefault() as Filter;
			if (selectedFilter != null && !string.IsNullOrEmpty(searchText))
			{
				// Mirror the results into the corresponding Filter object to allow the
				// RadioButton representation used when not snapped to reflect the change
				selectedFilter.Active = true;

				// TODO: Respond to the change in active filter by setting this.DefaultViewModel["Results"]
				//       to a collection of items with bindable Image, Title, Subtitle, and Description properties
				//var items = await Utilities.Search(selectedFilter.Description, searchText);

				this.DefaultViewModel["Results"] = selectedFilter.Results;

				// Ensure results are found
				if (selectedFilter.Count > 0)
				{
					VisualStateManager.GoToState(this, "ResultsFound", true);
					return;
				}
			}

			// Display informational text when there are no search results.
			VisualStateManager.GoToState(this, "NoResultsFound", true);
		}


		void ItemView_ItemClick(object sender, ItemClickEventArgs e)
		{
			// Navigate to the appropriate destination page, configuring the new page
			// by passing required information as a navigation parameter
			var itemId = ((SampleDataItem)e.ClickedItem).UniqueId;
			this.Frame.Navigate(typeof(ItemDetailPage), itemId);
		}

		/// <summary>
		/// Invoked when a filter is selected using a RadioButton when not snapped.
		/// </summary>
		/// <param name="sender">The selected RadioButton instance.</param>
		/// <param name="e">Event data describing how the RadioButton was selected.</param>
		void Filter_Checked(object sender, RoutedEventArgs e)
		{
			// Mirror the change into the CollectionViewSource used by the corresponding ComboBox
			// to ensure that the change is reflected when snapped
			if (filtersViewSource.View != null)
			{
				var filter = (sender as FrameworkElement).DataContext;
				filtersViewSource.View.MoveCurrentTo(filter);
			}
		}

		/// <summary>
		/// View model describing one of the filters available for viewing search results.
		/// </summary>
		private sealed class Filter : MyFirstApp.Common.BindableBase
		{
			private String _name;
			private int _count;
			private bool _active;

			public Filter(String name, IEnumerable<SampleDataItem> results, bool active = false)
			{
				this.Results = results;
				this.Name = name;
				this.Count = results == null ? 0 : results.Count();
				this.Active = active;
			}

			public override String ToString()
			{
				return Description;
			}

			public String Name
			{
				get { return _name; }
				set { if (this.SetProperty(ref _name, value)) this.OnPropertyChanged("Description"); }
			}

			public int Count
			{
				get { return _count; }
				set { if (this.SetProperty(ref _count, value)) this.OnPropertyChanged("Description"); }
			}

			public bool Active
			{
				get { return _active; }
				set { this.SetProperty(ref _active, value); }
			}

			public String Description
			{
				get { return String.Format("{0} ({1})", _name, _count); }
			}

			public IEnumerable<SampleDataItem> Results { get; set; }
		}
	}
}
